import json
from datetime import datetime, timedelta, timezone
import os
import requests

SLACK_SEND_MESSAGE_URL = "SLACK_SEND_MESSAGE_URL"
WATCHING_TOO_LONG_TIMER = "watchingTooLongTimer"
RESPONSE_STATUS_CODE = "status_code"

def get_slack_send_message_url(environ_name: str)->str:
    '''
    SlackのWeb Hook App用のURLを取得する
    
    parameter:
    environ_name 環境変数名

    return:
    取得したURL
    '''
    url = os.getenv(environ_name, "")
    return url

def is_slack_send_message_url_valid(url: str)->bool:
    '''
    Web Hook App用のURLが有効な文字列かをチェックする
    
    parameter:
    url Web Hook App用のURL
    
    return:
    有効な場合はTrue
    '''
    return type(url) is str and len(url) != 0

def get_local_datetime_string_from_timestamp(timestamp: int)->str:
    '''
    Timestamp値から日本のローカル時間に変換し、YYYY-MM-DDTHH:mm:ss 形式の文字列を返す 
    
    parameter:
    timestamp Timestamp値
    
    return:
    日本のローカルタイムに変換したYYYY-MM-DDTHH:mm:ss 形式の文字列
    '''
    JST = timezone(timedelta(hours=+9), 'JST')
    local_datetime = datetime.fromtimestamp(timestamp, tz=JST)
    local_datetime_string = local_datetime.strftime('%Y-%m-%dT%H:%M:%S')
    return local_datetime_string 

def get_watching_too_long_timer(event: dict)->str:
    '''
    eventからwatching_too_long_timerのTimestamp値を取得する
    
    parameter:
    event Lambdaに渡されるevent値
    
    return:
    watching_too_long_timerのTimestamp値
    '''
    watching_too_long_timer = event["payload"]["state"]["timers"][WATCHING_TOO_LONG_TIMER]
    return get_local_datetime_string_from_timestamp(watching_too_long_timer)

def build_message_text(watching_too_long_timer:str)->str:
    '''
    Slackへ書き込むメッセージを組み立てる
    
    parameter:
    watching_too_long_timer watching_too_long_timerの日本ローカル時間に変換した文字列
    
    return:
    組み立てた文字列
    '''
    return f"長い時間テレビを見すぎ！ {watching_too_long_timer} まで見てるよ！"

def send_slack_message(url:str, message: str)->object:
    '''
    Slackへメッセージを送信
    
    parameter:
    message 送信するメッセージ
    
    return:
    requestsへPOSTした結果のResponseオブジェクト
    '''
    body = { "text": message }
    headers = {'Content-type':'application/json'}
    try:
        response = requests.post(url, headers=headers, json=body)
    except requests.RequestException as e:
        print(e)
        raise e
    return response

def lambda_handler(event: dict, context: dict)->dict:
    '''
    Lambdaのイベントハンドラ
    
    parameter:
    event イベントパラメータ
    context Lambda Context
    
    return:
    テストのために取得した以下の値を返す
        requestsへPOSTした結果のresponseオブジェクトのテキスト
        送信先URL
        watching_too_long_timerの文字列
    '''
    slack_send_message_url = get_slack_send_message_url(SLACK_SEND_MESSAGE_URL)
    
    watching_too_long_timer = get_watching_too_long_timer(event)

    if is_slack_send_message_url_valid(slack_send_message_url):
        message = build_message_text(watching_too_long_timer)
        response = send_slack_message(slack_send_message_url, message)

    return json.dumps({
        SLACK_SEND_MESSAGE_URL: slack_send_message_url,
        WATCHING_TOO_LONG_TIMER: watching_too_long_timer,
        RESPONSE_STATUS_CODE: response.status_code
        })
