import json
import os
import pytest
import pytest_mock

from  deeplens_event_handler import app

SLACK_SEND_MESSAGE_URL = "SLACK_SEND_MESSAGE_URL"
WATCHING_TOO_LONG_TIMER = "watchingTooLongTimer"
RESPONSE_STATUS_CODE = "status_code"

@pytest.fixture()
def deeplens_event():
    """ Generate Deeplens Event """

    return {
        "eventTime": 1586560823340,
        "payload": {
            "actionExecutionId": "516e8e36-2c1f-3861-891f-75dc0acd0a1f",
            "detector": {
                "detectorModelName": "WatchingTVDetectorModel",
                "keyValue": "deeplens_qUEHPfaWSXyUwQEDfpwadQ",
                "detectorModelVersion": "7"
            },
            "eventTriggerDetails": {
                "inputName": "None",
                "messageId": "None",
                "triggerType": "Timer"
            },
            "state": {
                "stateName": "WatchingTooLong",
                "variables": {},
                "timers": {
                    "watchingTooLongTimer": 1586560822,
                    "notWatchingTimer": 1586560882
                }
            }
        },
        "eventName": "InvokeLamba"
    }

def test_get_slack_send_message_url():
    
    os.environ[SLACK_SEND_MESSAGE_URL] = "http://sample.com"

    ret_value = app.get_slack_send_message_url(SLACK_SEND_MESSAGE_URL)
    assert len(ret_value) != 0

def test_is_slack_send_message_url_valid():
    url = "https://samples.com"
    ret_value = app.is_slack_send_message_url_valid(url)
    assert ret_value == True

def test_get_local_datetime_string_from_timestamp():
    ret_value = app.get_local_datetime_string_from_timestamp(1586560822)
    assert "2020-04-11T08:20:22" == ret_value

def test_get_watching_too_long_timer(deeplens_event):
    ret_value = app.get_watching_too_long_timer(deeplens_event)
    assert "2020-04-11T08:20:22" == ret_value

def test_build_message_text():
    message = "長い時間テレビを見すぎ！ 2020-04-11T08:20:22 まで見てるよ！"
    ret_value = app.build_message_text("2020-04-11T08:20:22")
    assert message == ret_value

def test_send_slack_message(mocker):
    responseMock = mocker.Mock()
    responseMock.status_code = 200
    mocker.patch('requests.post').return_value = responseMock
    
    ret_value = app.send_slack_message("http://sample.com","hello")
    assert ret_value.status_code == 200 

def test_lambda_handler(deeplens_event, mocker):
    responseMock = mocker.Mock()
    responseMock.status_code = 200
    mocker.patch('requests.post').return_value = responseMock

    ret_value = app.lambda_handler(deeplens_event, "")
    data = json.loads(ret_value)
    assert data[SLACK_SEND_MESSAGE_URL] != None
    assert data[WATCHING_TOO_LONG_TIMER] == "2020-04-11T08:20:22"
    assert data[RESPONSE_STATUS_CODE] == 200
    